import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_webview_plugin/flutter_webview_plugin.dart';
import 'package:myfatoorah_flutter/model/MFError.dart';
import 'package:myfatoorah_flutter/model/initsession/SDKInitSessionResponse.dart';
import 'package:myfatoorah_flutter/utils/AppConstants.dart';
import 'package:myfatoorah_flutter/utils/ErrorsEnum.dart';
//
import '../myfatoorah_flutter.dart';
import 'MFApplePayButton.dart';

class MFApplePayButtonWebView extends State<MFApplePayButton> {
  static late MFExecutePaymentRequest request;
  static late String apiLang;
  static late Function callback;
  static double mHeight = 0;
  final flutterWebViewPlugin = FlutterWebviewPlugin();
  late StreamSubscription<double> _onProgressChanged;
  late StreamSubscription<WebViewStateChanged> _onStateChanged;

  MFApplePayButtonWebView(double height) {
    WidgetsFlutterBinding.ensureInitialized();
    mHeight = height;
  }

  void load(
      MFInitiateSessionResponse initiateSession,
      MFExecutePaymentRequest req,
      String lang,
      Function func
      ) {
    if (req.displayCurrencyIso == null || req.displayCurrencyIso!.isEmpty) {
      func(
          "",
          MFResult.fail<MFPaymentStatusResponse>(new MFError(
              ErrorHelper
                  .getValue(ErrorsEnum.APPLE_PAY_CURRENCY_ERROR)
                  .code,
              ErrorHelper
                  .getValue(ErrorsEnum.APPLE_PAY_CURRENCY_ERROR)
                  .message)));
      return;
    }

    var url = "https://ap.myfatoorah.com/?sessionId=${initiateSession.sessionId!}&countryCode=${initiateSession.countryCode!}&currencyCode=${req.displayCurrencyIso}&amount=${req.invoiceValue}&platform=flutter";
    print("url: $url");
    request = req;
    apiLang = lang;
    callback = func;
    flutterWebViewPlugin.reloadUrl(url);
  }
@override
  void initState() {
    super.initState();
    _onStateChanged = flutterWebViewPlugin.onStateChanged
        .listen((WebViewStateChanged state) {
      String url = state.url;
      print("#" + state.url);

      Uri uri = Uri.dataFromString(url);

      if (state.url.contains("finish")) {
        String? sessionId = uri.queryParameters["sessionId"];
        executePayment(context, sessionId!);
      }
    });
  }

  @override
  void dispose() {
    super.dispose();
    _onProgressChanged.cancel();
    _onStateChanged.cancel();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: mHeight,
      child: WebviewScaffold(
        resizeToAvoidBottomInset: true,
        url: "",
        withJavascript: true,
        // javascriptChannels: Set.from([
        //   JavascriptChannel(
        //       name: 'Success',
        //       onMessageReceived: (JavascriptMessage message) {
        //         executePayment(context, message.message);
        //       }
        //   )
        // ]),
        mediaPlaybackRequiresUserGesture: false,
        withZoom: false,
        withLocalStorage: true,
        hidden: false,
        scrollBar: false,
        clearCache: true,
        initialChild: Container(),
      ),
    );
  }

  void executePayment(BuildContext context, String sessionId) {
    request.sessionId = sessionId;
    MFSDK.callExecutePaymentForInAppApplePay(context, request, apiLang, callback);
  }

  void returnPaymentFailed(String error) {
    callback(
        "",
        MFResult.fail<MFPaymentStatusResponse>(new MFError(
            ErrorHelper
                .getValue(ErrorsEnum.EMBEDDED_PAYMENT_ERROR)
                .code,
            error)));
  }
  void checkCallBacks(String url) {
    print("#" + url);
    if ((url.contains(AppConstants.callBackUrl!) ||
        url.contains(AppConstants.errorUrl!))) {

      Uri uri = Uri.dataFromString(url);
      String? paymentId = uri.queryParameters["paymentId"];
      var request = MFPaymentStatusRequest(paymentId: paymentId);
    }
  }
}
